package com.smppw.analysis.application.service.info;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.application.dto.info.FundSimilarReq;
import com.smppw.analysis.application.dto.info.ManualFundNoticeReq;
import com.smppw.analysis.domain.dataobject.FundArchivesInfoDO;
import com.smppw.analysis.domain.dataobject.FundFeeDo;
import com.smppw.analysis.domain.dataobject.FundSimilarDo;
import com.smppw.analysis.domain.dataobject.ManualFundNoticeInfoDO;
import com.smppw.analysis.domain.dto.info.*;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.analysis.domain.service.NavService;
import com.smppw.analysis.infrastructure.utils.StringUtil;
import com.smppw.common.pojo.IStrategy;
import com.smppw.common.pojo.NewSubStrategy;
import com.smppw.common.pojo.dto.DateValue;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcTimeRangeDto;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.common.pojo.enums.NavType;
import com.smppw.common.pojo.enums.TimeRange;
import com.smppw.common.pojo.enums.Visibility;
import com.smppw.constants.DateConst;
import com.smppw.core.IndicatorService;
import com.smppw.utils.BigDecimalUtils;
import com.smppw.utils.CalcUtils;
import com.smppw.utils.NavUtil;
import com.smppw.utils.StrategyHandleUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.stereotype.Service;

import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class FundInfoService {
    private static final Map<String, String> ACCRUED_FREQUENCY = MapUtil.newHashMap();
    private static final Map<String, String> ACCRUED_METHOD = MapUtil.newHashMap();

    static {
        ACCRUED_FREQUENCY.put("1", "月");
        ACCRUED_FREQUENCY.put("2", "季");
        ACCRUED_FREQUENCY.put("3", "半年");
        ACCRUED_FREQUENCY.put("4", "年");
        ACCRUED_FREQUENCY.put("0", "不固定");
        ACCRUED_METHOD.put("1", "高水位");
        ACCRUED_METHOD.put("2", "高水位+赎回时补充");
        ACCRUED_METHOD.put("3", "单客户高水位");
    }

    private final NavService navService;
    private final BaseInfoService baseInfoService;
    private final BaseIndicatorServiceV2 baseIndicatorServiceV2;

    public FundInfoService(NavService navService,
                           BaseInfoService baseInfoService,
                           BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        this.navService = navService;
        this.baseInfoService = baseInfoService;
        this.baseIndicatorServiceV2 = baseIndicatorServiceV2;
    }

    /**
     * 私募基金相似产品，计算相关性时代码可以优化（参考基金相关性）
     *
     * @param req /
     * @return /
     */
    public List<FundSimilarVO> getFundSimilarList(FundSimilarReq req) {
        List<String> refIds = ListUtil.toLinkedList(req.getRefId());
        IStrategy strategy = StrategyHandleUtils.getStrategy(req.getStrategy());
        Map<String, List<DateValue>> secNavMap = this.navService.getSecIdDateValueNavListMapByDb(refIds,
                null, null, Visibility.Both, NavType.CumulativeNav);
        List<IndicatorCalcTimeRangeDto> secTimeRanges = IndicatorService.getInstance().getSecTimeRange(req.getRefId(), Frequency.Monthly, secNavMap);
        String startDate = null;
        String endDate = null;
        if (CollUtil.isNotEmpty(secTimeRanges)) {
            for (IndicatorCalcTimeRangeDto secTimeRange : secTimeRanges) {
                if (TimeRange.Last1Year == secTimeRange.getTimeRange()) {
                    startDate = secTimeRange.getStartDate();
                    endDate = secTimeRange.getEndDate();
                    break;
                }
            }
        }
        FundSimilarParams params = req.convert();
        params.setStartDate(startDate);
        List<FundSimilarDo> dataList = this.baseInfoService.getFundSimilarList(params);
        List<String> fundIds = dataList.stream().map(FundSimilarDo::getFundId).collect(Collectors.toList());
        if (CollUtil.isNotEmpty(fundIds)) {
            refIds.addAll(fundIds);
        }
        Map<String, List<DateValue>> allNavMap = this.navService.getSecIdDateValueNavListMapByDb(refIds,
                startDate, endDate, Visibility.Both, NavType.CumulativeNav);
        Map<String, List<IndicatorCalcPropertyDto>> multiSecRetListNew = this.baseIndicatorServiceV2.getMultiSecRetListNew(
                refIds, ListUtil.empty(), Frequency.Monthly, Frequency.Daily, startDate, endDate, false, params.getBenchmarkId(),
                params.getRaiseType(), strategy, Visibility.Both, NavType.CumulativeNav, allNavMap);
        List<Map<String, Object>> initRetList = NavUtil.handleNextStepNew(refIds, ListUtil.empty(), multiSecRetListNew, false);
        Map<String, Map<String, Double>> retMap = this.transfer2TwoDimensionMap(initRetList);
        Double[][] corArr = CalcUtils.getCorrelationMatrix(retMap, ListUtil.toList(params.getRefId()), refIds);
        List<FundSimilarVO> resultList = ListUtil.list(true);
        for (FundSimilarDo map : dataList) {
            FundSimilarVO similarVO = BeanUtil.copyProperties(map, FundSimilarVO.class);
            // 因为第一个基金是当前基金，所以需要 + 1 来取该基金与当前基金的相关性
            int i = fundIds.indexOf(map.getFundId()) + 1;
            similarVO.setSimilar(corArr[0][i]);
            similarVO.setStrategy(StrategyHandleUtils.getStrategy(map.getStrategy()).getStrategyNameDesc());
            similarVO.setSubStrategy(this.getStrategyName(map.getSubStrategy()));
            similarVO.setThirdStrategy(this.getStrategyName(map.getThirdStrategy()));
            resultList.add(similarVO);
        }
        return resultList.stream().filter(e -> e.getSimilar() != null && e.getSimilar() >= Double.parseDouble(params.getThreshold())).collect(Collectors.toList());
    }

    private String getStrategyName(String strategyKey) {
        IStrategy strategy = StrategyHandleUtils.getStrategy(strategyKey);
        if (strategy == null || strategy == NewSubStrategy.OtherStrategy || strategy.getStrategyId() == -1) {
            return "--";
        }
        return strategy.getStrategyNameDesc();
    }

    private Map<String, Map<String, Double>> transfer2TwoDimensionMap(List<Map<String, Object>> initRetList) {
        Map<String, Map<String, Double>> map = new TreeMap<>();
        for (Map<String, Object> unit : initRetList) {
            if (unit != null) {
                String id = unit.get("fund_id").toString();
                if (!map.containsKey(id)) {
                    map.put(id, new TreeMap<>());
                }
                Object t = unit.get("ret_range");
                if (t == null) {
                    continue;
                }
                map.get(id).put(unit.get("end_date").toString(), Double.valueOf(unit.get("ret_range").toString()));
            }

        }
        return map;
    }

    public List<ManualFundNoticeInfoVO> noticeList(ManualFundNoticeReq params) {
        Map<String, String> typeMapper = MapUtil.builder("1", "10").put("2", "20").put("3", "30").put("4", "50, 70, 90, 91").build();
        String fundId = params.getRefId();
        String title = params.getTitle();
        // type 逗号拆分 转list，直接数据库in查询
        List<String> typeList = ListUtil.list(false);
        if (CharSequenceUtil.isNotBlank(params.getType())) {
            String s = typeMapper.get(params.getType());
            typeList.addAll(StrUtil.split(s, ","));
        }
        //todo 确认基金公告表所在的库
        List<ManualFundNoticeInfoDO> dataList = this.baseInfoService.queryFundAnnounce(fundId, typeList, title);
        List<ManualFundNoticeInfoVO> resultList = ListUtil.list(true);
        // key value互换
        Map<String, String> reverse = MapUtil.reverse(typeMapper);
        for (ManualFundNoticeInfoDO temp : dataList) {
            ManualFundNoticeInfoVO vo = BeanUtil.copyProperties(temp, ManualFundNoticeInfoVO.class);
            vo.setType(reverse.getOrDefault(vo.getType(), "4"));
            vo.setDate(DateUtil.formatDate(temp.getDate()));
            resultList.add(vo);
        }
        return resultList;
    }

    public ManualFundInvestInfoVO investInfo(String refId) {
        ManualFundInvestInfoVO fundInvestInfoVO = new ManualFundInvestInfoVO();
        FundArchivesInfoDO fundArchivesInfo = this.baseInfoService.queryFundArchives(refId);
        if (fundArchivesInfo == null) {
            return fundInvestInfoVO;
        }
        fundInvestInfoVO.setInvestmentObjective(fundArchivesInfo.getInvestmentObjective());
        fundInvestInfoVO.setStrategyDescription(fundArchivesInfo.getStrategyDescription());
        fundInvestInfoVO.setInvestmentScope(fundArchivesInfo.getInvestmentScope());
        return fundInvestInfoVO;
    }

    public FundBaseFeeVO getFundFee(String refId) {
        FundFeeDo fundFee = this.baseInfoService.getFundFee(refId);
        DateFormatUtils.format(new Date(), DateConst.YYYY, TimeZone.getDefault());
        //数字格式处理
        FundFeeDo fundFees = numberFormat(fundFee);
        FundBaseFeeVO fee = BeanUtil.copyProperties(fundFees, FundBaseFeeVO.class);
        String frequency = Optional.ofNullable(fundFees.getAccruedFrequency()).map(e -> StrUtil.subBetween(e, "[", "]"))
                .map(e -> StrUtil.split(e, ",")).orElse(ListUtil.empty()).stream().filter(StrUtil::isNotBlank)
                .map(ACCRUED_FREQUENCY::get).collect(Collectors.joining(","));
        fee.setAccruedFrequency(frequency);
        String method = Optional.ofNullable(fundFees.getAccruedMethod()).map(e -> StrUtil.subBetween(e, "[", "]"))
                .map(e -> StrUtil.split(e, ",")).orElse(ListUtil.empty()).stream().filter(StrUtil::isNotBlank)
                .map(ACCRUED_METHOD::get).collect(Collectors.joining(","));
        fee.setAccruedMethod(method);
        return fee;
    }

    private FundFeeDo numberFormat(FundFeeDo fundFees) {
        FundFeeDo fundFeeDo = fundFees;
        String minInvestmentShare = fundFees.getMinInvestmentShare();
        fundFeeDo.setMinInvestmentShare(minInvestmentShare != null && StringUtil.isNumeric(minInvestmentShare) ? BigDecimalUtils.toBigDecimal(minInvestmentShare)
                .setScale(2, RoundingMode.HALF_DOWN) + "万" : minInvestmentShare);

        String managementFeeBank = fundFees.getManagementFeeBank();
        fundFeeDo.setManagementFeeBank(managementFeeBank != null && StringUtil.isNumeric(managementFeeBank) ? BigDecimalUtils.toBigDecimal(managementFeeBank)
                .setScale(2, RoundingMode.HALF_DOWN) + "%" : managementFeeBank);

        String subscriptionFee = fundFees.getSubscriptionFee();
        fundFeeDo.setSubscriptionFee(subscriptionFee != null && StringUtil.isNumeric(subscriptionFee) ? BigDecimalUtils.toBigDecimal(subscriptionFee)
                .setScale(2, RoundingMode.HALF_DOWN) + "%" : subscriptionFee);

        String redemptionFee = fundFees.getRedemptionFee();
        fundFeeDo.setRedemptionFee(redemptionFee != null && StringUtil.isNumeric(redemptionFee) ? BigDecimalUtils.toBigDecimal(redemptionFee)
                .setScale(2, RoundingMode.HALF_DOWN) + "%" : redemptionFee);

        String managementFee = fundFees.getManagementFee();
        fundFeeDo.setManagementFee(managementFee != null && StringUtil.isNumeric(managementFee) ? BigDecimalUtils.toBigDecimal(managementFee)
                .setScale(2, RoundingMode.HALF_DOWN) + "%" : managementFee);

        String managementFeeConsult = fundFees.getManagementFeeConsult();
        fundFeeDo.setManagementFeeConsult(managementFeeConsult != null && StringUtil.isNumeric(managementFeeConsult) ? BigDecimalUtils.toBigDecimal(managementFeeConsult)
                .setScale(2, RoundingMode.HALF_DOWN) + "%" : managementFeeConsult);

        String performanceFee = fundFees.getPerformanceFee();
        fundFeeDo.setPerformanceFee(performanceFee != null && StringUtil.isNumeric(performanceFee) ? BigDecimalUtils.toBigDecimal(performanceFee)
                .setScale(2, RoundingMode.HALF_DOWN) + "%" : performanceFee);

        String guardLine = fundFees.getGuardLine();
        fundFeeDo.setGuardLine(guardLine != null && StringUtil.isNumeric(guardLine) ? BigDecimalUtils.toBigDecimal(guardLine)
                .setScale(2, RoundingMode.HALF_DOWN) + "%" : guardLine);

        String subsequentInvestmentShare = fundFees.getSubsequentInvestmentShare();
        fundFeeDo.setSubsequentInvestmentShare(subsequentInvestmentShare != null && StringUtil.isNumeric(subsequentInvestmentShare) ? BigDecimalUtils.toBigDecimal(subsequentInvestmentShare)
                .setScale(2, RoundingMode.HALF_DOWN) + "万" : subsequentInvestmentShare);

        String stopLossLine = fundFees.getStopLossLine();
        fundFeeDo.setStopLossLine(stopLossLine != null && StringUtil.isNumeric(stopLossLine) ? BigDecimalUtils.toBigDecimal(stopLossLine)
                .setScale(2, RoundingMode.HALF_DOWN) + "%" : stopLossLine);
        return fundFeeDo;
    }

}
